package com.gmei.data.monitor.operator;

import com.alibaba.fastjson.JSONObject;
import com.gmei.data.monitor.cache.SimpleCacheService;
import com.gmei.data.monitor.callable.DeviceCallable;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.async.ResultFuture;
import org.apache.flink.streaming.api.functions.async.RichAsyncFunction;

import java.util.Arrays;
import java.util.Collections;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import static java.util.concurrent.Executors.newFixedThreadPool;

public class RichAsyncFunctionOperator extends RichAsyncFunction<JSONObject, JSONObject> {
    public static final SimpleCacheService<Integer, String> deviceCache = new SimpleCacheService<Integer, String>(2000, 10);

    public String inJdbcUrl;

    public RichAsyncFunctionOperator(String inJdbcUrl) {
        this.inJdbcUrl = inJdbcUrl;
    }

    @Override
    public void open(Configuration parameters) throws Exception {
        super.open(parameters);
        initTable();
    }

    @Override
    public void asyncInvoke(JSONObject jsonObject, ResultFuture<JSONObject> resultFuture) throws Exception {
        JSONObject appObject = jsonObject.getJSONObject("APP");
        if (null != appObject) {
            String appAction = appObject.getString("action");
            if (null != appAction) {
                String[] interact = {"like", "comment", "collect"};
                if (Arrays.asList(interact).contains(appAction)) {
                    Integer userId = appObject.getInteger("user_id");
                    if (userId != null) {
                        String deviceId = deviceCache.getValue(userId, new DeviceCallable(userId, inJdbcUrl));
                        if (deviceId != null) {
                            appObject.put("device_id",deviceId);
                        } else {
                            appObject.put("device_id","");
                        }
                        resultFuture.complete(Collections.singleton(jsonObject));
                    }
                }
            }
        }
    }

    public void initTable() throws Exception {
        ExecutorService es = newFixedThreadPool(10);
        Future deviceFuture = es.submit(new DeviceCallable(1, inJdbcUrl));
        String deviceId  = (String)deviceFuture.get();
        deviceCache.putValue(1,deviceId);
    }
}
